@@ -42,30 +42,33 @@
 
 #define REQUEST		(1 << 0)
 #define UINT8		(1 << 1)
-#define UINT16		(1 << 2)
-#define SINT16		(1 << 3)
-#define UINT32		(1 << 4)
-#define SINT32		(1 << 5)
-#define ADDRIPV4	(1 << 6)
-#define STRING		(1 << 7)
-#define ARRAY		(1 << 8)
-#define RFC3361		(1 << 9)
-#define RFC3397		(1 << 10)
-#define RFC3442		(1 << 11)
-#define RFC5969		(1 << 12)
-#define ADDRIPV6	(1 << 13)
-#define BINHEX		(1 << 14)
-#define FLAG		(1 << 15)
-#define NOREQ		(1 << 16)
-#define EMBED		(1 << 17)
-#define ENCAP		(1 << 18)
-#define INDEX		(1 << 19)
-#define OPTION		(1 << 20)
-#define DOMAIN		(1 << 21)
-#define ASCII		(1 << 22)
-#define RAW		(1 << 23)
-#define ESCSTRING	(1 << 24)
-#define ESCFILE		(1 << 25)
+#define INT8		(1 << 2)
+#define UINT16		(1 << 3)
+#define INT16		(1 << 4)
+#define UINT32		(1 << 5)
+#define INT32		(1 << 6)
+#define ADDRIPV4	(1 << 7)
+#define STRING		(1 << 8)
+#define ARRAY		(1 << 9)
+#define RFC3361		(1 << 10)
+#define RFC1035		(1 << 11)
+#define RFC3442		(1 << 12)
+#define OPTIONAL	(1 << 13)
+#define ADDRIPV6	(1 << 14)
+#define BINHEX		(1 << 15)
+#define FLAG		(1 << 16)
+#define NOREQ		(1 << 17)
+#define EMBED		(1 << 18)
+#define ENCAP		(1 << 19)
+#define INDEX		(1 << 20)
+#define OPTION		(1 << 21)
+#define DOMAIN		(1 << 22)
+#define ASCII		(1 << 23)
+#define RAW		(1 << 24)
+#define ESCSTRING	(1 << 25)
+#define ESCFILE		(1 << 26)
+#define BITFLAG		(1 << 27)
+#define RESERVED	(1 << 28)
 
 struct dhcp_opt {
 	uint32_t option; /* Also used for IANA Enterpise Number */
@@ -74,6 +77,7 @@ struct dhcp_opt {
 	char *var;
 
 	int index; /* Index counter for many instances of the same option */
+	char bitflags[8];
 
 	/* Embedded options.
 	 * The option code is irrelevant here. */
@@ -85,6 +89,7 @@ struct dhcp_opt {
 	size_t encopts_len;
 };
 
+const char *dhcp_get_hostname(char *, size_t, const struct if_options *);
 struct dhcp_opt *vivso_find(uint32_t, const void *);
 
 ssize_t dhcp_vendor(char *, size_t);
@@ -101,12 +106,9 @@ int make_option_mask(const struct dhcp_opt *, size_t,
     uint8_t *, const char *, int);
 
 size_t encode_rfc1035(const char *src, uint8_t *dst);
-ssize_t decode_rfc3397(char *, size_t, const uint8_t *, size_t);
+ssize_t decode_rfc1035(char *, size_t, const uint8_t *, size_t);
 ssize_t print_string(char *, size_t, int, const uint8_t *, size_t);
-ssize_t print_option(char *, size_t, int, const uint8_t *, size_t,
-    const char *);
-int dhcp_set_leasefile(char *, size_t, int,
-    const struct interface *, const char *);
+int dhcp_set_leasefile(char *, size_t, int, const struct interface *);
 
 size_t dhcp_envoption(struct dhcpcd_ctx *,
     char **, const char *, const char *, struct dhcp_opt *,
