@@ -1444,6 +1444,13 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 						    ifname);
 						goto err_sla;
 					}
+					if (sla->sla == 0) {
+						logger(ctx, LOG_WARNING,
+						    "%s: sla of 0 is not "
+						    "RFC3633 (section 12.1) "
+						    "compliant",
+						    ifname);
+					}
 				}
 				p = np;
 			}
@@ -1683,14 +1690,18 @@ err_sla:
 			t |= UINT8;
 		else if (strcasecmp(arg, "bitflags") == 0)
 			t |= BITFLAG;
+		else if (strcasecmp(arg, "uint8") == 0)
+			t |= UINT8;
+		else if (strcasecmp(arg, "int8") == 0)
+			t |= INT8;
 		else if (strcasecmp(arg, "uint16") == 0)
 			t |= UINT16;
 		else if (strcasecmp(arg, "int16") == 0)
-			t |= SINT16;
+			t |= INT16;
 		else if (strcasecmp(arg, "uint32") == 0)
 			t |= UINT32;
 		else if (strcasecmp(arg, "int32") == 0)
-			t |= SINT32;
+			t |= INT32;
 		else if (strcasecmp(arg, "flag") == 0)
 			t |= FLAG;
 		else if (strcasecmp(arg, "raw") == 0)
