@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -145,6 +145,23 @@ ipv4_iffindlladdr(struct interface *ifp)
 	return NULL;
 }
 
+static struct ipv4_addr *
+ipv4_iffindmaskaddr(struct interface *ifp, const struct in_addr *addr)
+{
+	struct ipv4_state *state;
+	struct ipv4_addr *ap;
+
+	state = IPV4_STATE(ifp);
+	if (state) {
+		TAILQ_FOREACH (ap, &state->addrs, next) {
+			if ((ap->addr.s_addr & ap->net.s_addr) ==
+			    (addr->s_addr & ap->net.s_addr))
+				return ap;
+		}
+	}
+	return NULL;
+}
+
 struct ipv4_addr *
 ipv4_findaddr(struct dhcpcd_ctx *ctx, const struct in_addr *addr)
 {
@@ -159,6 +176,20 @@ ipv4_findaddr(struct dhcpcd_ctx *ctx, const struct in_addr *addr)
 	return NULL;
 }
 
+struct ipv4_addr *
+ipv4_findmaskaddr(struct dhcpcd_ctx *ctx, const struct in_addr *addr)
+{
+	struct interface *ifp;
+	struct ipv4_addr *ap;
+
+	TAILQ_FOREACH(ifp, ctx->ifaces, next) {
+		ap = ipv4_iffindmaskaddr(ifp, addr);
+		if (ap)
+			return ap;
+	}
+	return NULL;
+}
+
 int
 ipv4_srcaddr(const struct rt *rt, struct in_addr *addr)
 {
@@ -366,7 +397,7 @@ ipv4_findrt(struct dhcpcd_ctx *ctx, const struct rt *rt, int flags)
 	TAILQ_FOREACH(r, ctx->ipv4_kroutes, next) {
 		if (rt->dest.s_addr == r->dest.s_addr &&
 #ifdef HAVE_ROUTE_METRIC
-		    rt->iface == r->iface &&
+		    (rt->iface == NULL || rt->iface == r->iface) &&
 		    (!flags || rt->metric == r->metric) &&
 #else
 		    (!flags || rt->iface == r->iface) &&
@@ -391,7 +422,7 @@ ipv4_freerts(struct rt_head *routes)
 /* If something other than dhcpcd removes a route,
  * we need to remove it from our internal table. */
 int
-ipv4_handlert(struct dhcpcd_ctx *ctx, int cmd, struct rt *rt)
+ipv4_handlert(struct dhcpcd_ctx *ctx, int cmd, const struct rt *rt, int flags)
 {
 	struct rt *f;
 
@@ -422,7 +453,8 @@ ipv4_handlert(struct dhcpcd_ctx *ctx, int cmd, struct rt *rt)
 		}
 		break;
 	}
-	return 0;
+
+	return flags ? 0 : ipv4ll_handlert(ctx, cmd, rt);
 }
 
 #define n_route(a)	 nc_route(NULL, a)
@@ -485,11 +517,6 @@ nc_route(struct rt *ort, struct rt *nrt)
 			logger(nrt->iface->ctx, LOG_ERR, "if_route (CHG): %m");
 	}
 
-	/* If the old route does not have an interface, give it the
-	 * interface of the new route for context. */
-	if (ort && ort->iface == NULL)
-		ort->iface = nrt->iface;
-
 #ifdef HAVE_ROUTE_METRIC
 	/* With route metrics, we can safely add the new route before
 	 * deleting the old route. */
@@ -843,15 +870,21 @@ ipv4_buildroutes(struct dhcpcd_ctx *ctx)
 		ipv4_freeroutes(dnr);
 	}
 
-	/* If we don't manage a default route, grab one without a
+	/* If there is no default route, grab one without a
 	 * gateway for any IPv4LL enabled interfaces. */
 	if (!has_default) {
-		TAILQ_FOREACH(ifp, ctx->ifaces, next) {
-			if ((rt = ipv4ll_default_route(ifp)) != NULL) {
-				if (ipv4_doroute(rt, nrs) == 1)
-					TAILQ_INSERT_TAIL(nrs, rt, next);
-				else
-					free(rt);
+		struct rt def;
+
+		memset(&def, 0, sizeof(def));
+		if (ipv4_findrt(ctx, &def, 0) == NULL) {
+			TAILQ_FOREACH(ifp, ctx->ifaces, next) {
+				if ((rt = ipv4ll_default_route(ifp)) != NULL) {
+					if (ipv4_doroute(rt, nrs) == 1) {
+						TAILQ_INSERT_TAIL(nrs, rt, next);
+						break;
+					} else
+						free(rt);
+				}
 			}
 		}
 	}
@@ -1166,6 +1199,13 @@ ipv4_applyaddr(void *arg)
 	 * notification right now via our link socket. */
 	if_initrt(ifp);
 	ipv4_buildroutes(ifp->ctx);
+	/* Announce the address */
+	if (ifo->options & DHCPCD_ARP) {
+		struct arp_state *astate;
+
+		if ((astate = arp_new(ifp, &state->addr)) != NULL)
+			arp_announce(astate);
+	}
 	if (state->state == DHS_BOUND) {
 		script_runreason(ifp, state->reason);
 		dhcpcd_daemonise(ifp->ctx);
