@@ -131,8 +131,7 @@ make_var(struct dhcpcd_ctx *ctx, const char *prefix, const char *var)
 	char *v;
 
 	len = strlen(prefix) + strlen(var) + 2;
-	v = malloc(len);
-	if (v == NULL) {
+	if ((v = malloc(len)) == NULL) {
 		logger(ctx, LOG_ERR, "%s: %m", __func__);
 		return NULL;
 	}
@@ -158,8 +157,10 @@ append_config(struct dhcpcd_ctx *ctx, char ***env, size_t *len,
 		eq = strchr(config[i], '=');
 		e1 = (size_t)(eq - config[i] + 1);
 		for (j = 0; j < *len; j++) {
-			if (strncmp(ne[j] + strlen(prefix) + 1,
-				config[i], e1) == 0)
+			if (strncmp(ne[j], prefix, strlen(prefix)) == 0 &&
+			    ne[j][strlen(prefix)] == '_' &&
+			    strncmp(ne[j] + strlen(prefix) + 1,
+			    config[i], e1) == 0)
 			{
 				p = make_var(ctx, prefix, config[i]);
 				if (p == NULL) {
@@ -239,7 +240,7 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 #endif
 #ifdef INET6
 	const struct dhcp6_state *d6_state;
-	int dhcp6, ra;
+	int static6, dhcp6, ra;
 #endif
 
 #ifdef INET
@@ -248,7 +249,7 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	istate = IPV4LL_CSTATE(ifp);
 #endif
 #ifdef INET6
-	dhcp6 = ra = 0;
+	static6 = dhcp6 = ra = 0;
 	d6_state = D6_CSTATE(ifp);
 #endif
 	if (strcmp(reason, "TEST") == 0) {
@@ -260,13 +261,15 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 			ra = 1;
 #endif
 #ifdef INET
-		else if (state->added)
-			dhcp = 1;
-		else
+		else if (istate && istate->addr.s_addr != INADDR_ANY)
 			ipv4ll = 1;
+		else
+			dhcp = 1;
 #endif
 	}
 #ifdef INET6
+	else if (strcmp(reason, "STATIC6") == 0)
+		static6 = 1;
 	else if (reason[strlen(reason) - 1] == '6')
 		dhcp6 = 1;
 	else if (strcmp(reason, "ROUTERADVERT") == 0)
@@ -359,6 +362,7 @@ make_env(const struct interface *ifp, const char *reason, char ***argv)
 	    || (ipv4ll && IPV4LL_STATE_RUNNING(ifp))
 #endif
 #ifdef INET6
+	    || (static6 && IPV6_STATE_RUNNING(ifp))
 	    || (dhcp6 && d6_state && d6_state->new)
 	    || (ra && ipv6nd_hasra(ifp))
 #endif
@@ -491,6 +495,20 @@ dumplease:
 	}
 #endif
 #ifdef INET6
+	if (static6) {
+		n = ipv6_env(NULL, NULL, ifp);
+		if (n > 0) {
+			nenv = realloc(env, sizeof(char *) *
+			    (elen + (size_t)n + 1));
+			if (nenv == NULL)
+				goto eexit;
+			env = nenv;
+			n = ipv6_env(env + elen, "new", ifp);
+			if (n == -1)
+				goto eexit;
+			elen += (size_t)n;
+		}
+	}
 	if (dhcp6 && D6_STATE_RUNNING(ifp)) {
 		n = dhcp6_env(NULL, NULL, ifp,
 		    d6_state->new, d6_state->new_len);
@@ -571,9 +589,9 @@ send_interface1(struct fd_list *fd, const struct interface *iface,
 	elen = (size_t)arraytostr((const char *const *)env, &s);
 	if ((ssize_t)elen == -1) {
 		free(s);
-		return -1;
-	}
-	retval = control_queue(fd, s, elen, 1);
+		retval = -1;
+	} else
+		retval = control_queue(fd, s, elen, 1);
 	ep = env;
 	while (*ep)
 		free(*ep++);
@@ -619,6 +637,10 @@ send_interface(struct fd_list *fd, const struct interface *ifp)
 #endif
 
 #ifdef INET6
+	if (IPV6_STATE_RUNNING(ifp)) {
+		if (send_interface1(fd, ifp, "STATIC6") == -1)
+			retval = -1;
+	}
 	if (RS_STATE_RUNNING(ifp)) {
 		if (send_interface1(fd, ifp, "ROUTERADVERT") == -1)
 			retval = -1;
